/*
 * Copyright (c) 2004-2020 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef _IBDIAGNET_H_
#define _IBDIAGNET_H_


#include <stdlib.h>

#include <infiniband/ibdiag/ibdiag.h>
#include <misc/cmdparser/cmdparser.h>

#include "ibdiagapp.h"

#include "ibdiagnet_types.h"
#include "ibdiagnet_stages.h"
#include "ibdiagnet_plugins.h"


/******************************************************/
class IBDiagnet : public IBDiagapp {
private:
    ////////////////////
    //members
    ////////////////////

    ////////////////////
    //methods
    ////////////////////

    //Returns: 0 - OK / 1 - error / 2 - OK but need to exit / 3 - parsing error
    virtual int HandleOption(string name, string value);
    virtual int PostParseFiles();

    virtual string GetToolName()
    {
        return IBDIAGNET_TOOL_NAME;
    }

    bool ParseRailOptions(const string& value);
    void ParseUnhealthyPortsOption(const string &value, string &switch_action, string &ca_action);

public:
    ////////////////////
    //methods
    ////////////////////
    IBDiagnet();
    virtual ~IBDiagnet();

    //Returns: 0 - OK / 1 - error
    virtual int Init();

    virtual Stage* CreateStage(uint8_t stageIndex);

public:
    class CmdLinePreprocessor {
        private:
            int argumentsCount;
            const char** p_argv;

        public:
            CmdLinePreprocessor() : argumentsCount(0), p_argv(NULL) {}
            ~CmdLinePreprocessor();

            int GetArgc() { return argumentsCount; }
            const char** GetArgv() { return p_argv; }
            int Init(int argc, const char** argv);
            int Compare(const char *p_source, const char *p_pattern);
    };
};


#endif          /* not defined _IBDIAGNET_H_ */
