/*
 *  Copyright (c) 2004-2021 Mellanox Technologies LTD. All rights reserved
 *  Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef _IBDIAGNET_DEBUG_WRAP_H_
#define _IBDIAGNET_DEBUG_WRAP_H_


#include <errno.h>
#include <sys/stat.h>

#include <infiniband/misc/tool_trace/tool_trace.h>

#include <infiniband/ibdm/OutputControl.h>

/******************************************************/
class DebugWrap {
	bool 		m_opened;
	std::string m_modules;
	std::string m_levels;


public:
    DebugWrap() : m_opened(false) {
        //the constructor is called before the function main begins.
        umask(0); //Any created files will have mode S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH (0666)
    }

    ~DebugWrap() {
    	CloseFile();
    }

    void CloseFile()
    {
    	if(m_opened) {
    		m_opened = false;
    		tt_log_destroy();
    	}

    }

    void OpenFile()
    {
    	CloseFile();

    	OutputControl::Properties properties(DEBUG_FILE);

    	if(!properties.is_valid())
    		return;

    	if(!properties.enabled())
    		return;

    	if(!OutputControl::CreateFolder(properties.path()))
    		return;

    	if(!m_levels.empty() && !m_modules.empty())
    	{
    	    std::string env = "module=" + m_modules + ";level=" + m_levels;
            if (setenv(TT_LOG_LEVEL_ENV_VARIABLE_NAME, env.c_str(), 1) < 0) {
                printf("-W- Failed to reset debug environment (%s) to '%s' with error: %s\n",
                       TT_LOG_LEVEL_ENV_VARIABLE_NAME, env.c_str(), strerror(errno));
            }
    	}

        if (tt_log_construct_v2(0xff,   /* need_to_flush */
                properties.path().c_str(), /* log file */
                DEFAULT_LOG_SIZE,       /* max size (in MB) */
                0)) {                   /* append to prev log */
            printf("-E- Failed to reopen log file=%s\n", DEBUG_FILE);
            exit(1);
        }

    	m_opened = true;
    }

    void set_modules(const std::string & modules) {
    	m_modules = modules;
    }

    void set_levels(const std::string & levels) {
    	m_levels = levels;
    }
};


#endif          /* not defined _IBDIAGNET_DEBUG_WRAP_H_ */
