/*
 * Copyright (c) 2004-2021 Mellanox Technologies LTD. All rights reserved.
 * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef _PHY_DIAG_TYPES_H_
#define _PHY_DIAG_TYPES_H_

#include "uphy/uphy.h"

/*****************************************************************************/
/* some defines */
#define TO_STR_(x)  #x
#define TO_STR(x)   TO_STR_(x)
#define FLT_FMT     "%.3f"

/*****************************************************************************/

enum PhyPluginSupportedNodesType {
    SUPPORT_CA                          = 0x01,
    SUPPORT_SW                          = 0x02,
    SUPPORT_RTR                         = 0x04,
    SUPPORT_SPECIAL_NODES               = 0x08,
    SUPPORT_SW_CA                       = SUPPORT_SW | SUPPORT_CA,
    SUPPORT_SW_CA_RTR                   = SUPPORT_SW | SUPPORT_CA | SUPPORT_RTR,
    SUPPORT_CA_AND_SPECIAL_NODES        = SUPPORT_CA | SUPPORT_SPECIAL_NODES,
    SUPPORT_ALL                         = SUPPORT_SW_CA_RTR | SUPPORT_SPECIAL_NODES
};

enum PPBMPMonitorGroup {
    PPBMP_RAW_BER_RS    = 0,
    PPBMP_RAW_BER_FC    = 1,
    PPBMP_EFFECTIVE_BER = 2,
    PPBMP_PREFEC_BER    = 4,
    PPBMP_SYMBOL_BER    = 5
};

bool IsPhyPluginSupportNodeType(PhyPluginSupportedNodesType supported_nodes, const IBNode* node);

#define PLUGIN_NAME             "Phy Diagnostic (Plugin)"
#define PLUGIN_DESC             "This plugin performs phy diagnostic."
#define NET_DUMP_EXT_FILE       "ibdiagnet2.net_dump_ext"
#define NET_DUMP_EXT_AGG_FILE   "ibdiagnet2.net_dump_ext_agg"
#define PHY_ACC_REG_CABLE_FILE  "ibdiagnet2.phy_acc_reg_cables"
#define BER_THRESHOLD_ERROR     (1e-8)
#define BER_THRESHOLD_WARNING   (1e-13)

// set max size of smp acc reg in dwords (44 / 4)
#define SMP_ACC_REG_TLV_LEN    11

/*****************************************************************************/
/* sections defines */
#define SECTION_PHY_CNTRS                        ACC_REG_PPCNT_PHYS_LAYER_CNTRS_INTERNAL_SECTION_NAME
#define SECTION_RAW_BER                          "PHY_DB2"
#define SECTION_EFF_BER                          "EFF_BER"

#define SECTION_PCI_CNTRS                        "P_DB1"
#define SECTION_PCI_TIMERS                       "P_DB6"
#define SECTION_PCI_LANES                        "P_DB7"

#define DD_PHY_TYPE                              1
#define DD_PCI_TYPE                              2

/*****************************************************************************/
/* checks defines */
#define CHECK_NAME_PHY_CNTRS_CHECK                      "DD checking"
#define CHECK_NAME_PHY_CNTRS_RESET(section_name)        section_name + " reseting"
#define CHECK_NAME_PHY_CNTRS_RETRIEVE                   "DD retrieving"
#define CHECK_NAME_RETRIEVE(section_name)               section_name + " retrieving"
#define CHECK_NAME_EFF_BER_CALC                         "Effective BER Check Calculated"
#define CHECK_NAME_EFF_BER                              "Effective BER Check"
#define CHECK_NAME_SYM_BER                              "Symbol BER Check"
#define CHECK_NAME_PCI_DEGRADATION                      "PCI degradation check"

/*****************************************************************************/
#define BER_THRESHOLD_TABLE_DEFAULT_STR         "(default)"

#define NO_DATA_VAL                             "N/A"

// PCI macros
#define MAX_PCI_IDX         4
#define MAX_PCI_DEPTH       4
#define MAX_PCI_NODES       16
// end point
#define PCI_PORT_TYPE_EP    0
// upstream
#define PCI_PORT_TYPE_US    5
// downstream
#define PCI_PORT_TYPE_DS    6


//phy data fields in order of appearance
typedef enum {
    EN_TIME_SINCE_LAST_CLEAR = 0,
    EN_SYMBOL_ERRORS,
    EN_SYNC_HEADERS_ERRORS,
    EN_EDPL_LANE0,
    EN_EDPL_LANE1,
    EN_EDPL_LANE2,
    EN_EDPL_LANE3,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE0,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE1,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE2,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE3,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE0,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE1,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE2,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE3,
    EN_RS_FEC_CORRECTED_BLOCKS,
    EN_RS_FEC_UNCORRECTABLE_BLOCKS,
    EN_RS_FEC_NO_ERRORS_BLOCKS,
    EN_RS_FEC_SINGLE_ERROR_BLOCKS,
    EN_RS_FEC_CORRECTED_SYMBOLS_TOTAL,
/*
rs_fec_corrected_symbols_lane0_high
A8h
rs_fec_corrected_symbols_lane0_low
ACh
rs_fec_corrected_symbols_lane1_high
B0h
rs_fec_corrected_symbols_lane1_low
B4h
rs_fec_corrected_symbols_lane2_high
B8h rs_fec_corrected_symbols_lane2_low BCh
rs_fec_corrected_symbols_lane3_high
C0h
rs_fec_corrected_symbols_lane3_low
C4h
link_down_events
C8h
successful_recovery_events*/
} EnPhyDataFields_t;

typedef enum {
    NOT_SUP_ACC_REG,
    VIA_SMP,
    VIA_GMP,
} AccRegVia_t;

enum CableModuleInfoIdentifier {
    CABLE_MODULE_INFO_IDENTIFIER_QSFP28             = 0,
    CABLE_MODULE_INFO_IDENTIFIER_QSFP_PLUS,
    CABLE_MODULE_INFO_IDENTIFIER_SFP28_SFP_plus,
    CABLE_MODULE_INFO_IDENTIFIER_QSA,
    CABLE_MODULE_INFO_IDENTIFIER_Backplane,
    CABLE_MODULE_INFO_IDENTIFIER_SFP_DD,
    CABLE_MODULE_INFO_IDENTIFIER_QSFP_DD,
    CABLE_MODULE_INFO_IDENTIFIER_QSFP_CMIS,
    CABLE_MODULE_INFO_IDENTIFIER_OSFP,
    CABLE_MODULE_INFO_IDENTIFIER_C2C,
    CABLE_MODULE_INFO_IDENTIFIER_DSFP,
    CABLE_MODULE_INFO_IDENTIFIER_QSFP_SPLIT_CABLE
};

#define UPHY_REGISTER_DUMP_SHORT    "short"
#define UPHY_REGISTER_DUMP_LONG     "long"
#define UPHY_REGISTER_DUMP_ALL      "all"

enum UPHYRegisterDump {
    UPHYRegisterDump_SHORT      = 0x1,
    UPHYRegisterDump_LONG       = 0x2,
    UPHYRegisterDump_ALL        = UPHYRegisterDump_SHORT | UPHYRegisterDump_LONG,
};

//how many lanes to check
#define LANE_NUM                                4
#define PCI_LANE_NUM                            16
//it's currently empirically determined
#define ZERO_RAW_BER_EXPONENT                       255

/* infiniband access register defines */
#define IB_ATTR_SMP_ACCESS_REG                      0xff52
#define IB_ATTR_GMP_ACCESS_REG                      0x0051
#define ACCESS_REGISTER_ID_SLRG                     0x5028
#define ACCESS_REGISTER_ID_SLRP                     0x5026
#define ACCESS_REGISTER_ID_SLTP                     0x5027
#define ACCESS_REGISTER_ID_SLSIR                    0x502C
#define ACCESS_REGISTER_ID_PTAS                     0x5029
#define ACCESS_REGISTER_ID_PDDR                     0x5031
#define ACCESS_REGISTER_ID_PMDR                     0x503C
#define ACCESS_REGISTER_ID_PPHCR                    0x503e
#define ACCESS_REGISTER_ID_PEUCG                    0x506C
#define ACCESS_REGISTER_ID_PEMI                     0x506e
#define ACCESS_REGISTER_ID_PPLL                     0x5030
#define ACCESS_REGISTER_ID_MPEIN                    0x9050
#define ACCESS_REGISTER_ID_MPIR                     0x9059
#define ACCESS_REGISTER_ID_PMCR                     0x5045
#define ACCESS_REGISTER_ID_PTYS                     0x5004
#define ACCESS_REGISTER_ID_PPSLS                    0x50e3
#define ACCESS_REGISTER_ID_MPSCR                    0x910F
#define ACCESS_REGISTER_ID_MTSR                     0x9121
#define ACCESS_REGISTER_ID_PLTC                     0x5046
#define ACCESS_REGISTER_ID_PAOS                     0x5006
#define ACCESS_REGISTER_ID_PCAM                     0x507f
#define ACCESS_REGISTER_ID_PPCNT                    0x5008
// fans registers
#define ACCESS_REGISTER_ID_MFCR                     0x9001
#define ACCESS_REGISTER_ID_MFSM                     0x9003
#define ACCESS_REGISTER_ID_MFSL                     0x9004
#define ACCESS_REGISTER_ID_FORE                     0x9007
#define ACCESS_REGISTER_ID_MFNR                     0x903b
// temperature registers
#define ACCESS_REGISTER_ID_MTCAP                    0x9009
#define ACCESS_REGISTER_ID_MTMP                     0x900a
#define ACCESS_REGISTER_ID_MTWE                     0x900b
// current/voltage register
#define ACCESS_REGISTER_ID_MVCAP                    0x902e
#define ACCESS_REGISTER_ID_MVCR                     0x900c
// power supplies register
#define ACCESS_REGISTER_ID_MSPS                     0x900d
// System General Information register
#define ACCESS_REGISTER_ID_MSGI                     0x9021

#define ACCESS_REGISTER_ID_SLRIP                    0x5057
#define ACCESS_REGISTER_ID_SLREG                    0x5042
#define ACCESS_REGISTER_ID_SLCCT                    0x503b
#define ACCESS_REGISTER_ID_SLLM                     0x505b
#define ACCESS_REGISTER_ID_PPAMP                    0x5025
#define ACCESS_REGISTER_ID_PRTL                     0x5014
#define ACCESS_REGISTER_ID_PPBMP                    0x5051
#define ACCESS_REGISTER_ID_MRCS                     0x9151
#define ACCESS_REGISTER_ID_MPCNT                    0x9051

/*Power Telemetry*/
#define ACCESS_REGISTER_ID_PPSLC                    0x50e2
#define ACCESS_REGISTER_ID_PPSLD                    0x50e4
#define ACCESS_REGISTER_ID_MLPC                     0x9166
#define ACCESS_REGISTER_ID_MPPGS                    0x9175
#define ACCESS_REGISTER_ID_MPPDR                    0x9177
#define ACCESS_REGISTER_ID_MPPGH                    0x9176
#define ACCESS_REGISTER_ID_PPSPC                    0x50ee
#define ACCESS_REGISTER_ID_PPSPI                    0x50ed
#define ACCESS_REGISTER_ID_PPSPGC                   0x50ef
#define ACCESS_REGISTER_ID_PPSLG                    0x5073
#define ACCESS_REGISTER_ID_MPPCR                    0x9173
#define ACCESS_REGISTER_ID_MPPGCR                   0x9174

/* EMAD fields definitions */
#define EMAD_OP_TLV_METHOD_QUERY                    0x1
//EMAD Class Types
#define EMAD_CLASS_ACC_REG                          0x1
//EMAD TLV Types
#define EMAD_TLV_TYPE_OPERATION                     0x1
#define EMAD_TLV_TYPE_REG                           0x3
//EMAD TLV Types length in dword (4 bytes)
#define EMAD_TLV_TYPE_OPERATION_LEN                 4

//SMP Access register types
#define ACC_REG_PNAT_IB_PORT                        0x1
#define ACC_REG_PNAT_OOB_PORT                       0x3

#define UNUSED_FIELDS_NUM                           -1

#define ACC_REG_ALL_NAME                            "all"
#define ACC_REG_ALL_PCI_NAME                        "all_pci"
#define ACC_REG_ALL_POWER_TELEMTRY                  "pt"
#define ACC_REG_ALL_PEMI_NAME                       "pemi"
#define ACC_REG_ALL_PDDR_NAME                       "pddr"
#define ACC_REG_ALL_MPCNT_NAME                      "mpcnt"
#define ACC_REG_ALL_PPCNT_NAME                      "ppcnt"
#define ACC_REG_ALL_PPSLG_NAME                      "ppslg"

//SLRG
#define ACC_REG_SLRG_INTERNAL_SECTION_NAME          "PHY_DB3"
#define ACC_REG_SLRG_PCI_INTERNAL_SECTION_NAME      "P_DB3"
#define ACC_REG_SLRG_EXTERNAL_SECTION_NAME          "PHY_PORT_GRADE"
#define ACC_REG_SLRG_NAME                           "slrg"
#define ACC_REG_SLRG28NM_FIELDS_NUM                 26
#define ACC_REG_SLRG16NM_FIELDS_NUM                 30
#define ACC_REG_SLRG7NM_FIELDS_NUM                  19
#define ACC_REG_SLRG5NM_FIELDS_NUM                  19
#define ACC_REG_SLRG_MAX_FIELDS_NUM                 ACC_REG_SLRG16NM_FIELDS_NUM

//SLTP
#define ACC_REG_SLTP_INTERNAL_SECTION_NAME          "PHY_DB5"
#define ACC_REG_SLTP_NAME                           "sltp"
#define ACC_REG_SLTP_PCI_INTERNAL_SECTION_NAME      "P_DB5"
#define ACC_REG_SLTP28NM_FIELDS_NUM                 20
#define ACC_REG_SLTP16NM_FIELDS_NUM                 26
#define ACC_REG_SLTP7NM_FIELDS_NUM                  18
#define ACC_REG_SLTP5NM_FIELDS_NUM                  26
#define ACC_REG_SLTP_MAX_FIELDS_NUM                 ACC_REG_SLTP16NM_FIELDS_NUM

//SLRP
#define ACC_REG_SLRP_INTERNAL_SECTION_NAME          "PHY_DB4"
#define ACC_REG_SLRP_PCI_INTERNAL_SECTION_NAME      "P_DB4"
#define ACC_REG_SLRP_NAME                           "slrp"
#define ACC_REG_SLRP28NM_FIELDS_NUM                 40
#define ACC_REG_SLRP16NM_FIELDS_NUM                 57
#define ACC_REG_SLRP7NM_FIELDS_NUM                  161
#define ACC_REG_SLRP5NM_FIELDS_NUM                  63
#define ACC_REG_SLRP_MAX_FIELDS_NUM                 ACC_REG_SLRP7NM_FIELDS_NUM

//PTAS
#define ACC_REG_PTAS_INTERNAL_SECTION_NAME          "PHY_DB6"
#define ACC_REG_PTAS_NAME                           "ptas"
#define ACC_REG_PTAS_FIELDS_NUM                     23

//PPLL
#define ACC_REG_PPLL_INTERNAL_SECTION_NAME          "PHY_DB7"
#define ACC_REG_PPLL_NAME                           "ppll"
#define ACC_REG_PPLL28NM_FIELDS_NUM                 42
#define ACC_REG_PPLL16NM_FIELDS_NUM                 69
#define ACC_REG_PPLL7NM_FIELDS_NUM                  38
#define ACC_REG_PPLL5NM_FIELDS_NUM                  20
#define ACC_REG_PPLL_MAX_FIELDS_NUM                 ACC_REG_PPLL16NM_FIELDS_NUM

//MPEIN
#define ACC_REG_MPEIN_INTERNAL_SECTION_NAME          "P_DB8"
#define ACC_REG_MPEIN_NAME                           "mpein"
#define ACC_REG_MPEIN_FIELDS_NUM                     19

//MPIR
#define ACC_REG_MPIR_INTERNAL_SECTION_NAME          "P_DB2"
#define ACC_REG_MPIR_NAME                           "mpir"
#define ACC_REG_MPIR_FIELDS_NUM                     10


// PDDR (DD)
#define SECTION_TROUBLESHOOTING_INFO                ACC_REG_PDDR_TROUBLESHOOTING_INFO_INTERNAL_SECTION_NAME
#define SECTION_OPERATION_INFO                      ACC_REG_PDDR_OPERATION_INFO_INTERNAL_SECTION_NAME
#define SECTION_PHY_INFO                            ACC_REG_PDDR_PHY_INFO_INTERNAL_SECTION_NAME
#define SECTION_MODULE_INFO                         "PHY_DB12"
#define SECTION_DD_CABLE_MODULE_INFO                "PHY_DD_CABLE_INFO"
#define SECTION_ACC_REG_CABLE_MODULE_INFO           "PHY_ACC_REG_CABLE_INFO"
#define SECTION_LINK_DOWN_INFO                      ACC_REG_PDDR_LINK_DOWN_INFO_INTERNAL_SECTION_NAME
#define SECTION_LINK_UP_INFO                        ACC_REG_PDDR_LINKUP_INTERNAL_SECTION_NAME
#define SECTION_LATCHED_FLAG_INFO                   "PHY_DB25"
#define CABLE_MODULE_INFO_MLNX_OUI                                0x2c9
#define CABLE_MODULE_INFO_TYPE_COPPER_UNEQUALIZED                 0xa
#define CABLE_MODULE_INFO_TYPE_COPPER_PASSIVE_EQUALIZED           0xb
#define CABLE_MODULE_INFO_NO_SEPARABLE_CONNECTOR                  0x23
#define CABLE_MODULE_INFO_MIN_SUPPORT_TEMPERATURE                 -40
#define CABLE_MODULE_INFO_MAX_SUPPORT_TEMPERATURE                 125
#define CABLE_MODULE_INFO_CMIS_LENGTH_MUL_0_1                     0
#define CABLE_MODULE_INFO_CMIS_LENGTH_MUL_1                       0x1
#define CABLE_MODULE_INFO_CMIS_LENGTH_MUL_10                      0x2
#define CABLE_MODULE_INFO_CMIS_LENGTH_MUL_100                     0x3

// PPCNT (DD)
#define SECTION_RS_FEC_HISTOGRAM                    ACC_REG_PPCNT_RS_FEC_HISTOGRAMS_COUNTERS_INTERNAL_SECTION_NAME
#define SECTION_PLR_COUNTERS                        ACC_REG_PPCNT_PLR_COUNTERS_INTERNAL_SECTION_NAME
#define SECTION_PHY_LAYER_STATISTICS                ACC_REG_PPCNT_PHYS_LAYER_STAT_CNTRS_INTERNAL_SECTION_NAME
#define SECTION_INFINIBAND_GENERAL_COUNTERS         ACC_REG_PPCNT_INFINIBAND_GENERAL_COUNTERS_INTERNAL_SECTION_NAME
#define SECTION_GENERAL_COUNTERS                    ACC_REG_PPCNT_L1_GENERAL_COUNTERS_INTERNAL_SECTION_NAME
#define SECTION_PACKET_TYPE_COUNTERS                "PHY_DB39"

#define ACC_REG_SLRIP_INTERNAL_SECTION_NAME         "PHY_DB18"
#define ACC_REG_SLRIP_NAME                          "slrip"
#define ACC_REG_SLRIP_PCI_INTERNAL_SECTION_NAME     "P_DB18"
#define ACC_REG_SLRIP16NM_FIELDS_NUM                27
#define ACC_REG_SLRIP7NM_FIELDS_NUM                 91
#define ACC_REG_SLRIP5NM_FIELDS_NUM                 89
#define ACC_REG_SLRIP_MAX_FIELDS_NUM                ACC_REG_SLRIP7NM_FIELDS_NUM

#define ACC_REG_SLREG_INTERNAL_SECTION_NAME         "PHY_DB19"
#define ACC_REG_SLREG_NAME                          "slreg"
#define ACC_REG_SLREG16NM_FIELDS_NUM                29
#define ACC_REG_SLREG7NM_FIELDS_NUM                 48
#define ACC_REG_SLREG_MAX_FIELDS_NUM                ACC_REG_SLREG7NM_FIELDS_NUM

#define ACC_REG_SLCCT_INTERNAL_SECTION_NAME         "PHY_DB20"
#define ACC_REG_SLCCT_NAME                          "slcct"
#define ACC_REG_SLCCT_FIELDS_NUM                    37

#define ACC_REG_PPAMP_INTERNAL_SECTION_NAME         "PHY_DB21"
#define ACC_REG_PPAMP_NAME                          "ppamp"
#define ACC_REG_PPAMP_FIELDS_NUM                    18

#define SECTION_PAGE_IDENTIFICATION                 "PHY_DB22"

#define ACC_REG_SLLM_INTERNAL_SECTION_NAME          "PHY_DB23"
#define ACC_REG_SLLM_NAME                           "sllm"
#define ACC_REG_SLLM16NM_FIELDS_NUM                 19
#define ACC_REG_SLLM7NM_FIELDS_NUM                  22
#define ACC_REG_SLLM_MAX_FIELDS_NUM                 ACC_REG_SLLM7NM_FIELDS_NUM

//PDDR - Layouts
#define ACC_REG_PDDR_MODULE_INTERNAL_SECTION_NAME   "PHY_DB101"
#define ACC_REG_PDDR_MODULE_NAME                    "pddr_module"
#define ACC_REG_PDDR_MODULE_FIELDS_NUM              DIAGNOSTIC_DATA_MODULE_INFO_NUM_FIELDS
#define ACC_REG_PDDR_MODULE_HEADER                  ""

#define ACC_REG_PDDR_LINKUP_INTERNAL_SECTION_NAME  "PHY_DB24"
#define ACC_REG_PDDR_LINKUP_NAME                   "pddr_lup"
#define ACC_REG_PDDR_LINKUP_FIELDS_NUM             10
#define ACC_REG_PDDR_LINKUP_HEADER                 ""

#define ACC_REG_PDDR_LATCHED_INTERNAL_SECTION_NAME  "PHY_DB102"
#define ACC_REG_PDDR_LATCHED_NAME                   "pddr_lfi"
#define ACC_REG_PDDR_LATCHED_FIELDS_NUM             DIAGNOSTIC_DATA_LATCHED_FLAG_INFO_NUM_FIELDS
#define ACC_REG_PDDR_LATCHED_HEADER                 ""

#define ACC_REG_PDDR_PHY_INFO_INTERNAL_SECTION_NAME "PHY_DB11"
#define ACC_REG_PDDR_PHY_INFO_NAME                  "pddr_phy"
#define ACC_REG_PDDR_PHY_INFO_FIELDS_NUM            105
#define ACC_REG_PDDR_PHY_INFO_HEADER                ""

// PDDR Troubleshooting Info Register
#define ACC_REG_PDDR_TROUBLESHOOTING_INFO_INTERNAL_SECTION_NAME  "PHY_DB9"
#define ACC_REG_PDDR_TROUBLESHOOTING_INFO_NAME                   "pddr_tsi"
#define ACC_REG_PDDR_TROUBLESHOOTING_INFO_FIELDS_NUM             5
#define ACC_REG_PDDR_TROUBLESHOOTING_INFO_HEADER                 ""
#define ACC_REG_PDDR_TROUBLESHOOTING_INFO_PAGE_NUM               1

// PDDR Operation Info Register
#define ACC_REG_PDDR_OPERATION_INFO_INTERNAL_SECTION_NAME  "PHY_DB10"
#define ACC_REG_PDDR_OPERATION_INFO_NAME                   "pddr_oi"
#define ACC_REG_PDDR_OPERATION_INFO_FIELDS_NUM             31
#define ACC_REG_PDDR_OPERATION_INFO_HEADER                 ""
#define ACC_REG_PDDR_OPERATION_INFO_PAGE_NUM               0

//PDDR Link Down Info Register
#define ACC_REG_PDDR_LINK_DOWN_INFO_INTERNAL_SECTION_NAME  "PHY_DB13"
#define ACC_REG_PDDR_LINK_DOWN_INFO_NAME                   "pddr_ldi"
#define ACC_REG_PDDR_LINK_DOWN_INFO_FIELDS_NUM             38
#define ACC_REG_PDDR_LINK_DOWN_INFO_HEADER                 ""
#define ACC_REG_PDDR_LINK_DOWN_INFO_PAGE_NUM               6

//PEMI - Layouts
#define ACC_REG_PEMI_MODULE_S_INTERNAL_SECTION_NAME "PHY_DB110"
#define ACC_REG_PEMI_MODULE_S_NAME                  "pemi_module_s"
#define ACC_REG_PEMI_MODULE_S_FIELDS_NUM            35
#define ACC_REG_PEMI_MODULE_S_HEADER                ""

#define ACC_REG_PEMI_SNR_S_INTERNAL_SECTION_NAME    "PHY_DB111"
#define ACC_REG_PEMI_SNR_S_NAME                     "pemi_snr_s"
#define ACC_REG_PEMI_SNR_S_FIELDS_NUM               16
#define ACC_REG_PEMI_SNR_S_HEADER                   ""

#define ACC_REG_PEMI_LASER_S_INTERNAL_SECTION_NAME  "PHY_DB112"
#define ACC_REG_PEMI_LASER_S_NAME                   "pemi_laser_s"
#define ACC_REG_PEMI_LASER_S_FIELDS_NUM             18
#define ACC_REG_PEMI_LASER_S_HEADER                 ""

#define ACC_REG_PEMI_PAM4_S_INTERNAL_SECTION_NAME   "PHY_DB113"
#define ACC_REG_PEMI_PAM4_S_NAME                    "pemi_pam4_s"
#define ACC_REG_PEMI_PAM4_S_FIELDS_NUM              16
#define ACC_REG_PEMI_PAM4_S_HEADER                  ""

#define ACC_REG_PEMI_BER_S_INTERNAL_SECTION_NAME    "PHY_DB114"
#define ACC_REG_PEMI_BER_S_NAME                     "pemi_ber_s"
#define ACC_REG_PEMI_BER_S_FIELDS_NUM               8
#define ACC_REG_PEMI_BER_S_HEADER                   ""

#define ACC_REG_PEMI_FERC_S_INTERNAL_SECTION_NAME   "PHY_DB115"
#define ACC_REG_PEMI_FERC_S_NAME                    "pemi_ferc_s"
#define ACC_REG_PEMI_FERC_S_FIELDS_NUM              8
#define ACC_REG_PEMI_FERC_S_HEADER                  ""

#define ACC_REG_PEMI_MODULE_P_INTERNAL_SECTION_NAME "PHY_DB126"
#define ACC_REG_PEMI_MODULE_P_NAME                  "pemi_module_p"
#define ACC_REG_PEMI_MODULE_P_FIELDS_NUM            21
#define ACC_REG_PEMI_MODULE_P_HEADER                ""

#define ACC_REG_PEMI_SNR_P_INTERNAL_SECTION_NAME    "PHY_DB127"
#define ACC_REG_PEMI_SNR_P_NAME                     "pemi_snr_p"
#define ACC_REG_PEMI_SNR_P_FIELDS_NUM               5
#define ACC_REG_PEMI_SNR_P_HEADER                   ""

#define ACC_REG_PEMI_LASER_P_INTERNAL_SECTION_NAME  "PHY_DB128"
#define ACC_REG_PEMI_LASER_P_NAME                   "pemi_laser_p"
#define ACC_REG_PEMI_LASER_P_FIELDS_NUM             13
#define ACC_REG_PEMI_LASER_P_HEADER                 ""

#define ACC_REG_PEMI_PAM4_P_INTERNAL_SECTION_NAME   "PHY_DB129"
#define ACC_REG_PEMI_PAM4_P_NAME                    "pemi_pam4_p"
#define ACC_REG_PEMI_PAM4_P_FIELDS_NUM              5
#define ACC_REG_PEMI_PAM4_P_HEADER                  ""

#define ACC_REG_PEMI_BER_P_INTERNAL_SECTION_NAME    "PHY_DB130"
#define ACC_REG_PEMI_BER_P_NAME                     "pemi_ber_p"
#define ACC_REG_PEMI_BER_P_FIELDS_NUM               13
#define ACC_REG_PEMI_BER_P_HEADER                   ""

#define ACC_REG_PEMI_FERC_P_INTERNAL_SECTION_NAME   "PHY_DB131"
#define ACC_REG_PEMI_FERC_P_NAME                    "pemi_ferc_p"
#define ACC_REG_PEMI_FERC_P_FIELDS_NUM              1
#define ACC_REG_PEMI_FERC_P_HEADER                  ""

// PPCNT - layouts
#define ACC_REG_PPCNT_PLR_COUNTERS_INTERNAL_SECTION_NAME "PHY_DB15"
#define ACC_REG_PPCNT_PLR_COUNTERS_NAME                  "ppcnt_plr"
#define ACC_REG_PPCNT_PLR_COUNTERS_FIELDS_NUM            20
#define ACC_REG_PPCNT_PLR_COUNTERS_HEADER                ""

#define ACC_REG_PPCNT_PACKET_TYPE_COUNTERS_INTERNAL_SECTION_NAME   "PHY_DB391"
#define ACC_REG_PPCNT_PACKET_TYPE_COUNTERS_NAME                    "ppcnt_ptc"
#define ACC_REG_PPCNT_PACKET_TYPE_COUNTERS_FIELDS_NUM              12
#define ACC_REG_PPCNT_PACKET_TYPE_COUNTERS_HEADER                  ""

#define ACC_REG_PPCNT_INFINIBAND_GENERAL_COUNTERS_INTERNAL_SECTION_NAME "PHY_DB37"
#define ACC_REG_PPCNT_INFINIBAND_GENERAL_COUNTERS_NAME                  "ppcnt_ib_gen_cntrs"
#define ACC_REG_PPCNT_INFINIBAND_GENERAL_COUNTERS_FIELDS_NUM            20
#define ACC_REG_PPCNT_INFINIBAND_GENERAL_COUNTERS_HEADER                ""


#define ACC_REG_PPCNT_RS_FEC_HISTOGRAMS_COUNTERS_INTERNAL_SECTION_NAME "PHY_DB14"
#define ACC_REG_PPCNT_RS_FEC_HISTOGRAMS_COUNTERS_NAME                  "ppcnt_rsfec"
#define ACC_REG_PPCNT_RS_FEC_HISTOGRAMS_COUNTERS_FIELDS_NUM            21
#define ACC_REG_PPCNT_RS_FEC_HISTOGRAMS_COUNTERS_HEADER                ""

#define ACC_REG_PPCNT_PHYS_LAYER_CNTRS_INTERNAL_SECTION_NAME "PHY_DB1"
#define ACC_REG_PPCNT_PHYS_LAYER_CNTRS_NAME                  "ppcnt_plc"
#define ACC_REG_PPCNT_PHYS_LAYER_CNTRS_FIELDS_NUM            50
#define ACC_REG_PPCNT_PHYS_LAYER_CNTRS_HEADER                ""

#define ACC_REG_PPCNT_PHYS_LAYER_STAT_CNTRS_INTERNAL_SECTION_NAME "PHY_DB16"
#define ACC_REG_PPCNT_PHYS_LAYER_STAT_CNTRS_NAME                  "ppcnt_plsc"
#define ACC_REG_PPCNT_PHYS_LAYER_STAT_CNTRS_FIELDS_NUM            63
#define ACC_REG_PPCNT_PHYS_LAYER_STAT_CNTRS_HEADER                ""

#define ACC_REG_PPCNT_L1_GENERAL_COUNTERS_INTERNAL_SECTION_NAME "PHY_DB60"
#define ACC_REG_PPCNT_L1_GENERAL_COUNTERS_NAME                  "ppcnt_gen_cntrs"
#define ACC_REG_PPCNT_L1_GENERAL_COUNTERS_FIELDS_NUM            49
#define ACC_REG_PPCNT_L1_GENERAL_COUNTERS_HEADER                ""

// Power Telemtry
#define ACC_REG_PPSLC_INTERNAL_SECTION_NAME    "PHY_DB44"
#define ACC_REG_PPSLC_NAME                     "ppslc"
#define ACC_REG_PPSLC_FIELDS_NUM               29
#define ACC_REG_PPSLC_HEADER                   ""

#define ACC_REG_PPSLD_INTERNAL_SECTION_NAME "PHY_DB45"
#define ACC_REG_PPSLD_NAME                  "ppsld"
#define ACC_REG_PPSLD_FIELDS_NUM            5
#define ACC_REG_PPSLD_HEADER                ""

#define ACC_REG_MLPC_INTERNAL_SECTION_NAME "PHY_DB46"
#define ACC_REG_MLPC_NAME                  "mlpc"
#define ACC_REG_MLPC_FIELDS_NUM            21
#define ACC_REG_MLPC_HEADER                ""

#define ACC_REG_MPPDR_INTERNAL_SECTION_NAME "PHY_DB47"
#define ACC_REG_MPPDR_NAME                  "mppdr"
#define ACC_REG_MPPDR_FIELDS_NUM            64
#define ACC_REG_MPPDR_HEADER                ""

#define ACC_REG_MPPGH_INTERNAL_SECTION_NAME "PHY_DB48"
#define ACC_REG_MPPGH_NAME                  "mppgh"
#define ACC_REG_MPPGH_FIELDS_NUM            17
#define ACC_REG_MPPGH_HEADER                ""

#define ACC_REG_MPPGS_INTERNAL_SECTION_NAME "PHY_DB49"
#define ACC_REG_MPPGS_NAME                  "mppgs"
#define ACC_REG_MPPGS_FIELDS_NUM            4
#define ACC_REG_MPPGS_HEADER                ""





#define ACC_REG_PPSPC_INTERNAL_SECTION_NAME "PHY_DB51"
#define ACC_REG_PPSPC_NAME                  "ppspc"
#define ACC_REG_PPSPC_FIELDS_NUM            18
#define ACC_REG_PPSPC_HEADER                ""

#define ACC_REG_PPSPI_INTERNAL_SECTION_NAME "PHY_DB52"
#define ACC_REG_PPSPI_NAME                  "ppspi"
#define ACC_REG_PPSPI_FIELDS_NUM            40
#define ACC_REG_PPSPI_HEADER                ""

#define ACC_REG_PPSPGC_INTERNAL_SECTION_NAME "PHY_DB53"
#define ACC_REG_PPSPGC_NAME                  "ppspgc"
#define ACC_REG_PPSPGC_FIELDS_NUM            7
#define ACC_REG_PPSPGC_HEADER                ""

//PPSLG - layouts
#define ACC_REG_PPSLG_L1_CAPABILITIES_AND_STATUS_INTERNAL_SECTION_NAME "PHY_DB54"
#define ACC_REG_PPSLG_L1_CAPABILITIES_AND_STATUS_NAME                  "ppslg_cas"
#define ACC_REG_PPSLG_L1_CAPABILITIES_AND_STATUS_FIELDS_NUM            15
#define ACC_REG_PPSLG_L1_CAPABILITIES_AND_STATUS_HEADER                ""

#define ACC_REG_PPSLG_L1_CONFIGURATION_INTERNAL_SECTION_NAME "PHY_DB55"
#define ACC_REG_PPSLG_L1_CONFIGURATION_NAME                  "ppslg_config"
#define ACC_REG_PPSLG_L1_CONFIGURATION_FIELDS_NUM            3
#define ACC_REG_PPSLG_L1_CONFIGURATION_HEADER                ""

#define ACC_REG_PPSLG_L1_DEBUG_PAGE_INTERNAL_SECTION_NAME "PHY_DB56"
#define ACC_REG_PPSLG_L1_DEBUG_PAGE_NAME                  "ppslg_dbg"
#define ACC_REG_PPSLG_L1_DEBUG_PAGE_FIELDS_NUM            19
#define ACC_REG_PPSLG_L1_DEBUG_PAGE_HEADER                ""

#define ACC_REG_PPSLG_L1_STATISTICAL_COUNTERS_INTERNAL_SECTION_NAME "PHY_DB57"
#define ACC_REG_PPSLG_L1_STATISTICAL_COUNTERS_NAME                  "ppslg_stat_cntrs"
#define ACC_REG_PPSLG_L1_STATISTICAL_COUNTERS_FIELDS_NUM            18
#define ACC_REG_PPSLG_L1_STATISTICAL_COUNTERS_HEADER                ""

#define ACC_REG_MPPGCR_INTERNAL_SECTION_NAME "PHY_DB58"
#define ACC_REG_MPPGCR_NAME                  "mppgcr"
#define ACC_REG_MPPGCR_FIELDS_NUM            12
#define ACC_REG_MPPGCR_HEADER                ""

#define ACC_REG_MPPCR_INTERNAL_SECTION_NAME "PHY_DB59"
#define ACC_REG_MPPCR_NAME                  "mppcr"
#define ACC_REG_MPPCR_FIELDS_NUM            35
#define ACC_REG_MPPCR_HEADER                ""

// MPCNT - layouts
#define ACC_REG_MPCNT_PCI_CNT_INTERNAL_SECTION_NAME   "P_DB1"
#define ACC_REG_MPCNT_PCI_CNT_NAME                    "mpcnt_pci_cnt"
#define ACC_REG_MPCNT_PCI_CNT_FIELDS_NUM              20
#define ACC_REG_MPCNT_PCI_CNT_HEADER                  ""

#define ACC_REG_MPCNT_PCI_LCNT_INTERNAL_SECTION_NAME  "P_DB7"
#define ACC_REG_MPCNT_PCI_LCNT_NAME                   "mpcnt_pci_lcnt"
#define ACC_REG_MPCNT_PCI_LCNT_FIELDS_NUM             18
#define ACC_REG_MPCNT_PCI_LCNT_HEADER                 ""

#define ACC_REG_MPCNT_PCI_TIMERS_INTERNAL_SECTION_NAME  "P_DB6"
#define ACC_REG_MPCNT_PCI_TIMERS_NAME                   "mpcnt_pci_timers"
#define ACC_REG_MPCNT_PCI_TIMERS_FIELDS_NUM             23
#define ACC_REG_MPCNT_PCI_TIMERS_HEADER                 ""

#define ACC_REG_MPCNT_PCI_PHYS_INTERNAL_SECTION_NAME  "P_DB9"
#define ACC_REG_MPCNT_PCI_PHYS_NAME                   "mpcnt_pci_phys"
#define ACC_REG_MPCNT_PCI_PHYS_FIELDS_NUM             4
#define ACC_REG_MPCNT_PCI_PHYS_HEADER                 ""


// RESERVED up to DB132

//PMDR
#define ACC_REG_PMDR_INTERNAL_SECTION_NAME          "PHY_DB26"
#define ACC_REG_PMDR_NAME                           "pmdr"
#define ACC_REG_PMDR_FIELDS_NUM                     119
#define ACC_REG_PMDR_FIELDS_HEADER                  ""

//PPHCR
#define ACC_REG_PPHCR_INTERNAL_SECTION_NAME          "PHY_DB27"
#define ACC_REG_PPHCR_NAME                           "pphcr"
#define ACC_REG_PPHCR_FIELDS_NUM                     44

// SLSIR
#define ACC_REG_SLSIR_INTERNAL_SECTION_NAME         "PHY_DB28"
#define ACC_REG_SLSIR_NAME                          "slsir"
#define ACC_REG_SLSIR7NM_FIELDS_NUM                 44
#define ACC_REG_SLSIR5NM_FIELDS_NUM                 44
#define ACC_REG_SLSIR_MAX_FIELDS_NUM                std::max(ACC_REG_SLSIR7NM_FIELDS_NUM, \
                                                             ACC_REG_SLSIR5NM_FIELDS_NUM)

// PMCR
#define ACC_REG_PMCR_INTERNAL_SECTION_NAME          "PHY_DB29"
#define ACC_REG_PMCR_NAME                           "pmcr"
#define ACC_REG_PMCR_FIELDS_NUM                     45

// PTYS
#define ACC_REG_PTYS_INTERNAL_SECTION_NAME          "PHY_DB30"
#define ACC_REG_PTYS_NAME                           "ptys"
#define ACC_REG_PTYS_FIELDS_NUM                     43

// PPSLS
#define ACC_REG_PPSLS_INTERNAL_SECTION_NAME         "PHY_DB40"
#define ACC_REG_PPSLS_NAME                          "ppsls"
#define ACC_REG_PPSLS_FIELDS_NUM                    6

// MTSR
#define ACC_REG_MTSR_INTERNAL_SECTION_NAME          "PHY_DB41"
#define ACC_REG_MTSR_NAME                           "mtsr"
#define ACC_REG_MTSR_FIELDS_NUM                     4

// PLTC
#define ACC_REG_PLTC_INTERNAL_SECTION_NAME          "PHY_DB42"
#define ACC_REG_PLTC_NAME                           "pltc"
#define ACC_REG_PLTC_FIELDS_NUM                     15

// PAOS
#define ACC_REG_PAOS_INTERNAL_SECTION_NAME          "PHY_DB43"
#define ACC_REG_PAOS_NAME                           "paos"
#define ACC_REG_PAOS_FIELDS_NUM                     16


// MPSCR
#define ACC_REG_MPSCR_INTERNAL_SECTION_NAME          "PHY_DB50"
#define ACC_REG_MPSCR_NAME                           "mpscr"
#define ACC_REG_MPSCR_FIELDS_NUM                     6

//PEUCG
#define ACC_REG_PEUCG_VER_INTERNAL_SECTION_NAME     "PHY_DB31"
#define ACC_REG_PEUCG_DLN_INTERNAL_SECTION_NAME     "PHY_DB32"
#define ACC_REG_PEUCG_CLN_INTERNAL_SECTION_NAME     "PHY_DB33"

//PCAM
#define ACC_REG_PCAM_INTERNAL_SECTION_NAME          "PHY_DB34"
#define ACC_REG_PCAM_NAME                           "pcam"
#define ACC_REG_PCAM_FIELDS_NUM                     4
#define ACC_REG_PCAM_FIELDS_HEADER                  ""

//PRTL
#define ACC_REG_PRTL_INTERNAL_SECTION_NAME          "PHY_DB35"
#define ACC_REG_PRTL_NAME                           "prtl"
#define ACC_REG_PRTL_FIELDS_NUM                     9
#define ACC_REG_PRTL_FIELDS_HEADER                  ""

// PPBMP
#define ACC_REG_PPBMP_INTERNAL_SECTION_NAME          "PHY_DB36"
#define ACC_REG_PPBMP_NAME                           "ppbmp"
#define ACC_REG_PPBMP_FIELDS_NUM                     5
#define ACC_REG_PPBMP_FIELDS_HEADER                  ""

// MRCS
#define ACC_REG_MRCS_INTERNAL_SECTION_NAME           "PHY_DB38"
#define ACC_REG_MRCS_NAME                            "mrcs"
#define ACC_REG_MRCS_FIELDS_NUM                      8
#define ACC_REG_MRCS_FIELDS_HEADER                   ""


#define ACC_REG_PEUCG_NAME                          "peucg"
#define ACC_REG_PEUCG_FIELDS_NUM                    153

// MFCR
#define ACC_REG_MFCR_INTERNAL_SECTION_NAME          "AVAILABLE_FANS"
#define ACC_REG_MFCR_NAME                           "mfcr"
// MFSM
#define ACC_REG_MFSM_INTERNAL_SECTION_NAME          "FANS_SPEED"
#define ACC_REG_MFSM_NAME                           "mfsm"
// MFSL
#define ACC_REG_MFSL_INTERNAL_SECTION_NAME          "FANS_THRESHOLDS"
#define ACC_REG_MFSL_NAME                           "mfsl"
// FORE
#define ACC_REG_FORE_INTERNAL_SECTION_NAME          "FANS_ALERT"
#define ACC_REG_FORE_NAME                           "fore"
// MFNR
#define ACC_REG_MFNR_INTERNAL_SECTION_NAME          "FANS_SERIAL_NUMBER"
#define ACC_REG_MFNR_NAME                           "mfnr"
// MTCAP
#define ACC_REG_MTCAP_INTERNAL_SECTION_NAME         "NUM_OF_TEMP_SENSORS"
#define ACC_REG_MTCAP_NAME                          "mtcap"
// MTMP
#define ACC_REG_MTMP_INTERNAL_SECTION_NAME          "TEMPERATURE_SENSORS"
#define ACC_REG_MTMP_NAME                           "mtmp"
// MTWE
#define ACC_REG_MTWE_INTERNAL_SECTION_NAME          "TEMPERATURE_SENSORS_ALERT"
#define ACC_REG_MTWE_NAME                           "mtwe"
// MVCAP
#define ACC_REG_MVCAP_INTERNAL_SECTION_NAME         "AVAILABLE_POWER_SENSORS"
#define ACC_REG_MVCAP_NAME                          "mvcap"
// MVCR
#define ACC_REG_MVCR_INTERNAL_SECTION_NAME          "POWER_SENSORS"
#define ACC_REG_MVCR_NAME                           "mvcr"
// MSPS
#define ACC_REG_MSPS_INTERNAL_SECTION_NAME          "POWER_SUPPLIES"
#define ACC_REG_MSPS_NAME                           "msps"
// MSGI
#define ACC_REG_MSGI_INTERNAL_SECTION_NAME          "SYSTEM_GENERAL_INFORMATION"
#define ACC_REG_MSGI_NAME                           "msgi"

#define PORT_LANE_HEADER                            "NodeGuid,PortGuid,PortNum,Lane"
#define NODE_HEADER                                 "NodeGuid"
#define PORT_HEADER                                 "NodeGuid,PortGuid,PortNum"
#define PORT_INDEX_HEADER                           "NodeGuid,PortGuid,PortNum,Index"
#define GROUP_HEADER                                "NodeGuid"
#define NODE_SENSOR_HEADER                          "NodeGuid,SensorIndex"
#define TYPE_GROUP_HEADER                           "NodeGuid,Type,Group,Index,NumOfIndices"
#define PCI_DPN_HEADER                              "NodeGuid,PCIIndex,Depth,PCINode"
#define PCI_PORT_LANE_HEADER                        "NodeGuid,LocalPort,Lane"

#define SOCKET_DIRECT_TABLE_SECTION_NAME            "SOCKET_DIRECT"
#define SOCKET_DIRECT_TABLE_SECTION_HEADER          "NodeGuid,PCIIndex,Depth,PCINode,DeviceStatus"

#define UPHY_VERSION_INFO_SECTION_NAME              "UPHY_VERSION_INFO"
#define UPHY_VERSION_INFO_SECTION_HEADER            "NodeGuid,CLN_Version,CLN_JSON,DLN_Version,DLN_JSON"

// fans
#define AVAILABLE_FANS                              ",AvailableFans"
#define FANS_SPEED_HEADER                           ",FanSpeed"
#define FANS_THRESHOLDS_HEADER                      ",MinSpeed,MaxSpeed"
#define FANS_ALERT_HEADER                           ",FansUnderLimit,FansOverLimit"
#define FANS_NUMBERS_HEADER                         ",SerialNumber"
// temperature
#define TEMPERATURE_SENSORS_COUNT                   ",TemperatureSensorsCount"
#define TEMPERATURE_SENSORS_HEADER                  ",SensorName,Temperature,MaxTemperature" \
                                                    ",LowThreshold,HighThreshold"
#define TEMPERATURE_SENSORS_ALERT_HEADER            ",SensorsOverThreshold"
// power sensors
#define AVAILABLE_POWER_SENSORS                     ",AvailablePowerSensors"
#define POWER_SENSORS_HEADER                        ",SensorName,Voltage,Current"
// power supplies
#define POWER_SUPPLIES_HEADER                       ",PSUIndex,IsPresent,IsFRU,ACInput,DCState" \
                                                    ",AlertState,FanState,TemperatureState," \
                                                    "PowerCap,PowerConsumption,SerialNumber"
// System General Information
#define SYSTEM_GENERAL_INFORMATION_HEADER           ",SerialNumber,PartNumber,Revision,ProductName"


#define MAX_FANS                                    16
#define MAX_VOLTAGE_CURRENT_SENSORS                 64

#define SLRIP_IB_SEL_NUM                            4
#define SLREG_EIDX_NUM                              3
#define SLCCT_CYCLES_NUM                            2
#define SLCCT_CTLE_ARR_SIZE                         8

#define MAD_STATUS_UNSUP_REGISTER_GMP               (5 << 2)

#define PPAMP_IDX_DATA_SIZE                         16
#define MAX_OPAMP_GROUP_TYPE                        3

#define PAGE_IDENTIFICATION_DD_IDX                  0

/*****************************************************************************/

/* options defines */
#define OPTION_PHY_INFO_GET "get_phy_info"
#define OPTION_PHY_INFO_GET_SHORT ' '
#define OPTION_PHY_INFO_GET_ARG_DESC ""
#define OPTION_PHY_INFO_GET_DESC "Query all ports for phy "  \
    "information."

#define OPTION_PHY_CABLE_DISCONNECTED "phy_cable_disconnected"
#define OPTION_PHY_CABLE_DISCONNECTED_SHORT ' '
#define OPTION_PHY_CABLE_DISCONNECTED_ARG_DESC ""
#define OPTION_PHY_CABLE_DISCONNECTED_DESC "Query disconnected ports for phy cable info, " \
                                           "works with --get_phy_info."

#define OPTION_PPAMP_GET "get_ppamp"
#define OPTION_PPAMP_GET_SHORT ' '
#define OPTION_PPAMP_GET_ARG_DESC ""
#define OPTION_PPAMP_GET_DESC "Query all ports for PPAMP "  \
    "(Port Phy opAMP data), works with --get_phy_info."

#define OPTION_SHOW_CAP_REG "show_cap_reg"
#define OPTION_SHOW_CAP_REG_SHORT ' '
#define OPTION_SHOW_CAP_REG_ARG_DESC ""
#define OPTION_SHOW_CAP_REG_DESC "Dump capability registers (hidden "  \
    "by default), works with --get_phy_info."

#define OPTION_PHY_INFO_RESET "reset_phy_info"
#define OPTION_PHY_INFO_RESET_SHORT ' '
#define OPTION_PHY_INFO_RESET_ARG_DESC ""
#define OPTION_PHY_INFO_RESET_DESC "Query and clear all ports phy information."


#define OPTION_PHY_CABLE_FULL_DATA "phy_cable_full_data"
#define OPTION_PHY_CABLE_FULL_DATA_SHORT ' '
#define OPTION_PHY_CABLE_FULL_DATA_ARG_DESC ""
#define OPTION_PHY_CABLE_FULL_DATA_DESC "Send DD pages Module Info and Latch Info"  \
    "for ports already collected by cable plugin"

#define OPTION_PHY_POWER_TELEMETRY_REG "power_telemetry"
#define OPTION_PHY_POWER_TELEMETRY_REG_SHORT ' '
#define OPTION_PHY_POWER_TELEMETRY_REG_ARG_DESC ""
#define OPTION_PHY_POWER_TELEMETRY_REG_DESC "Send Power Telemetry Registers"

#define OPTION_PCI_INFO_GET "get_p_info"
#define OPTION_PCI_INFO_GET_SHORT ' '
#define OPTION_PCI_INFO_GET_ARG_DESC ""
#define OPTION_PCI_INFO_GET_DESC "Query all ports for PCI information."

#define OPTION_PCI_INFO_RESET "reset_p_info"
#define OPTION_PCI_INFO_RESET_SHORT ' '
#define OPTION_PCI_INFO_RESET_ARG_DESC ""
#define OPTION_PCI_INFO_RESET_DESC "Query and clear all ports PCI information."

#define OPTION_SHOW_BER_WARNINGS "show_ber_warnings"
#define OPTION_SHOW_BER_WARNINGS_SHORT ' '
#define OPTION_SHOW_BER_WARNINGS_ARG_DESC ""
#define OPTION_SHOW_BER_WARNINGS_DESC "Enable show warnings for BER Check in PHY plugin"

#define OPTION_BER_THRESHOLD_TABLE "ber_threshold_table"
#define OPTION_BER_THRESHOLD_TABLE_SHORT ' '
#define OPTION_BER_THRESHOLD_TABLE_ARG_DESC "<path_ber_threshold_table>"
#define OPTION_BER_THRESHOLD_TABLE_DESC "Load BER threshold table for errors, warnings and normals\n" \
                                        "Format file:\n" \
                                        "<Technology>:<Speed>:<FEC_Mode_ID>:" \
                                        "<BER_type> = <error_value>,<warning_value>,<normal_value>"

#define OPTION_CREATE_BER_THRESHOLD_TABLE "create_ber_threshold_table"
#define OPTION_CREATE_BER_THRESHOLD_TABLE_SHORT ' '
#define OPTION_CREATE_BER_THRESHOLD_TABLE_ARG_DESC "<path_ber_threshold_table>"
#define OPTION_CREATE_BER_THRESHOLD_TABLE_DESC "Generate BER threshold table for errors, warnings"  \
                                                " and normals."

#define OPTION_ENABLED_REGS "enabled_regs"
#define OPTION_ENABLED_REGS_SHORT ' '
#define OPTION_ENABLED_REGS_ARG_DESC "<registers_list>"
#define OPTION_ENABLED_REGS_DESC "Query Registers by given list"

#define OPTION_DISABLED_REGS "disabled_regs"
#define OPTION_DISABLED_REGS_SHORT ' '
#define OPTION_DISABLED_REGS_ARG_DESC "<registers_list>"
#define OPTION_DISABLED_REGS_DESC "Query Registers by given list"

#define OPTION_SLRG_REG "slrg"
#define OPTION_SLRG_REG_SHORT ' '
#define OPTION_SLRG_REG_ARG_DESC ""
#define OPTION_SLRG_REG_DESC "Query SLRG register, works with --get_phy_info."

#define OPTION_USE_ACC_REG "use_acc_reg"
#define OPTION_USE_ACC_REG_SHORT ' '
#define OPTION_USE_ACC_REG_ARG_DESC ""
#define OPTION_USE_ACC_REG_DESC "Query Registers insetad of Diagnostic Data, works with --get_phy_info, --get_p_info"

#define OPTION_ACC_REG_PRIORITY "acc_reg_priority"
#define OPTION_ACC_REG_PRIORITY_SHORT ' '
#define OPTION_ACC_REG_PRIORITY_ARG_DESC "<smp|gmp>"
#define OPTION_ACC_REG_PRIORITY_DESC "Choose priority for sending access registers"

#define OPTION_UPHY                 "uphy"
#define OPTION_UPHY_SHORT           ' '
#define OPTION_UPHY_ARG_DESC        "<uphy_file_list>"
#define OPTION_UPHY_DESC            "Query UPHY Registers by given list of UPHY Json files."

#define OPTION_UPHY_DUMPS           "uphy_dumps"
#define OPTION_UPHY_DUMPS_SHORT     ' '
#define OPTION_UPHY_DUMPS_ARG_DESC  "<dumps>"
#define OPTION_UPHY_DUMPS_DESC      "List of dumps for UPHY registers filter."


class Register;
class PHYNodeData : public PluginData
{
public:
    class UPHYData
    {
        class Element {

        public:
            const UPHY::Version*   version;
            const UPHY::DataSet*   dataset;

            Element() : version(NULL), dataset(NULL) {}

            ~Element() {
                delete version;
            }
        };

    public:
        Element cln;
        Element dln;
    };

public:
    UPHYData*                                   uphy; // this is a pointer because of the periodic run.
    pcam_reg*                                   pcam;
    std::bitset<256>                            nsb_bitset;
    std::set< std::pair<Register*, APort*> >    planarized_hca_per_reg_sent;
    u_int8_t                                    curr_power_profile_id;

    // Add clear data here in case if need for new iteration (for periodic run)
    void clean() {
        delete uphy;
        uphy = NULL;

        delete pcam;
        pcam = NULL;

        curr_power_profile_id = 0;

        nsb_bitset.reset();

        planarized_hca_per_reg_sent.clear();
    }

    PHYNodeData() : uphy(NULL), pcam(NULL), curr_power_profile_id(0) { }

    virtual ~PHYNodeData() {
        clean();
    }
};


class PHYPortData : public PluginData
{
public:
    typedef uint32_t pemi_group_cap_mask_t;
    pemi_group_cap_mask_t pemi_group_cap_mask;

    // Add clear data here in case if need for new iteration (for periodic run)
    void clean() {
    }

    PHYPortData() : pemi_group_cap_mask(std::numeric_limits<pemi_group_cap_mask_t>::max()) { }

    virtual ~PHYPortData() {
        clean();
    }
};

/******************************************************/
/** NSB Automation **/
/******************************************************/
#define NOT_SUPPORT_DIAGNOSTIC_DATA                 1
#define NOT_SUPPORT_SMP_ACCESS_REGISTER             2
#define NOT_SUPPORT_GMP_ACCESS_REGISTER             3
#define NOT_SUPPORT_NSB_LAST                        NOT_SUPPORT_GMP_ACCESS_REGISTER

/******************************************************/
class NSB {
    public:
        template<typename T>
            static uint64_t get(T * obj) {
                static uint64_t value = next();
                return value;
            }

    private:
        static uint64_t next() {
            static uint64_t value = NOT_SUPPORT_NSB_LAST;
            return ++value;
        }
};
/******************************************************/

#endif //_PHY_DIAG_TYPES_H_
