 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <limits>

#include "uphy_dataset.h"

#define UPFY_FIELD_DELIMETER "."

namespace UPHY
{
    const DataSet::Register * DataSet::add(const Register * reg)
    {
        if( reg == nullptr )
            return nullptr;

        if( m_registers_index[ reg->address() ] != nullptr )
            return nullptr;

        m_registers_index[ reg->address() ] = reg;

        return reg;
    }


    const DataSet::Enumerator * DataSet::add(const Enumerator * enumerator)
    {
        if( enumerator == nullptr )
            return nullptr;

        auto x = m_enums.insert(std::make_pair(enumerator->name(), enumerator));

        return x.second ? enumerator : nullptr;
    }

    const DataSet::variant_collection_t & DataSet::variants(const std::list<std::string> & dumps) const
    {
        // Build key
        std::stringstream ss;

        for( auto & x : dumps)
            ss << '.' << x;

        auto k = ss.str();

        // if variant already exist
        const auto x = m_variant_index.find( k );

        if( x != m_variant_index.end() )
            return x->second;

        auto & v = m_variant_index[ k ];

        // Builds new vatiant
        v.clear();
        v.reserve(1024);

        for( auto & x : m_registers_index ) // registers list loop
        {
            bool                    should_be_add   = dumps.empty();
            register_unit_flags_t   units           = RegisterUnitFlags_None;

            for( auto & d : dumps ) // dumps cmd args loop
                if( x.second->dumps().find(d) != x.second->dumps().end() )
                {
                    should_be_add = true;

                    if( d.rfind("rx_", 0) == 0 )
                    {
                        if( x.second->units() & RegisterUnitFlags_RX )
                            units = (register_unit_flags_t) (units | RegisterUnitFlags_RX);
                        else
                            ; // TODO::May be add error ???
                    }

                    if( d.rfind("tx_", 0) == 0 )
                    {
                        if( x.second->units() & RegisterUnitFlags_TX )
                            units = (register_unit_flags_t) (units | RegisterUnitFlags_TX);
                        else
                            ; // TODO::May be add error ???
                    }

                    if( d.rfind("pll_", 0) == 0 )
                    {
                        if( x.second->units() & RegisterUnitFlags_CLM )
                            units = (register_unit_flags_t) (units | RegisterUnitFlags_CLM);
                        else
                            ; // TODO::May be add error ???
                    }
                }

            if( should_be_add )
            {
                // in case if units was not declared by dumps filters we should use value of register
                if( units == RegisterUnitFlags_None )
                    units = x.second->units();

                if( units == RegisterUnitFlags_None )
                    v.push_back( Variant(*x.second, RegisterUnitFlags_None) );
                else
                {
                    // can replaced to lopp with shift bit
                    if( units & RegisterUnitFlags_CLM )
                        v.push_back( Variant(*x.second, RegisterUnitFlags_CLM) );

                    if( units & RegisterUnitFlags_RX )
                        v.push_back( Variant(*x.second, RegisterUnitFlags_RX) );

                    if( units & RegisterUnitFlags_TX )
                        v.push_back( Variant(*x.second, RegisterUnitFlags_TX) );
                }
            }
        }

        return v;
    }
}
